if (Redmine::VERSION::MAJOR > 2) || (Redmine::VERSION::MAJOR == 2 && Redmine::VERSION::MINOR >= 3)
  require_dependency 'issue_query'
else
  require_dependency 'query'
end

module GeomapsExtraQueries
  module GeomapsIssueQueryPatch
    # Called init init.rb
    def self.included(base)
      base.send(:include, GeoMapsInstanceMethods)

      base.class_eval do
        @errorMessage = nil
        alias_method_chain :initialize_available_filters, :state_change_to
        alias_method_chain :build_from_params, :state_change_to
        alias_method_chain :validate_query_filters, :state_change_to
      end
    end

    module GeoMapsInstanceMethods
      attr_writer :date_from, :date_to, :state_value

      #
      # initialize_available_filters
      #
      def initialize_available_filters_with_state_change_to
        initialize_available_filters_without_state_change_to
       
        # Ajouter entre sinon on ne peut pas mettre 3 paramètres
        operators["state_changed_between"] = :label_between        
        operators_by_filter_type[:state_change_to] = ["state_changed_between"]

        # Ajouter le filtre state_change_to de type state_change_to
        add_available_filter('state_change_to', type: :state_change_to)
      end

      #
      # validate_query_filters
      #
      # Vérifier les valeurs des paramètres state_change_to
      def build_from_params_with_state_change_to(params)
        res = build_from_params_without_state_change_to(params)

        fields = params[:fields] || params[:f]
        op     = params[:operators] || params[:op]
        value  = params[:values] || params[:v]

        #Rails.logger.info("===========================================================================================================")
        #Rails.logger.info("============================================= geomaps= plugin =============================================")
        #Rails.logger.info("= build_from_params_with_state_change_to = params = #{params}")
        #Rails.logger.info("=                                   res = #{res}")
        #Rails.logger.info("-----------------------------------------------------------------------------------------------------------")

        if op != nil and op.key?(:state_change_to) and value != nil and value.key?(:state_change_to)
          bOk, errorMessage, date_begin, date_end, state_value = isStateChangeToValid?(value[:state_change_to])
          if !bOk
            @geomapsErrorMessage = errorMessage
          end
        end
        res
      end

      #
      # validate_query_filters
      #
      def validate_query_filters_with_state_change_to
        res = validate_query_filters_without_state_change_to
        
        if @geomapsErrorMessage != nil
            add_filter_error(:state_change_to, @geomapsErrorMessage)
        end
        res
      end

      ## ------------------------------------------------------
      ##
      ## Fonction à mettre dans le helper mais je n'arrive pas
      ## à le charger, donc je laisse ici (hélas)
      ##
      ## ------------------------------------------------------

      # Retourne 5 valeurs :
      # true / false selon si ok ou pas
      # le message d'erreur si false
      # les paramètres int pour la valeur de l'état, Date (debut) et Date (fin)
      def isStateChangeToValid?(value)
        bRet = false
        errorMessageEntry = nil
        state_value = nil
        date_begin = nil
        date_end = nil

        if value.kind_of?(Array) and value.length == 3
          if value[0] == nil || value[0].length == 0
            errorMessageEntry = "date_start_not_filled"
          elsif value[1] == nil || value[1].length == 0
            errorMessageEntry = "date_end_not_filled"
          elsif value[2] == nil || value[2].length == 0
            errorMessageEntry = "state_not_filled"
          else
            date_begin  = convert_to_date(value[0])
            date_end    = convert_to_date(value[1])
            state_value = convert_to_int(value[2])
            if (date_begin == nil)
              errorMessageEntry = "date_start_bad_format"
            elsif (date_end == nil)
              errorMessageEntry = "date_end_bad_format"
            elsif (date_begin > date_end)
              errorMessageEntry = "date_begin_must_be_before_date_end"
            elsif (state_value == nil)
              errorMessageEntry = "state_bad_format"
            else
              bRet = true
            end
          end
        else
          errorMessageEntry = "filter_not_filled"
        end

        return bRet, errorMessageEntry, date_begin, date_end, state_value
      end

      def convert_to_date(a_string)
          Date.parse(a_string)
        rescue
          nil
      end

      def convert_to_int(a_string)
        a_string.to_i
        rescue
          nil
      end
    end
  end
end